
#include "fNIRS_EVAL_v4_functions.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <util/delay.h>

void Timer_init() {		
	OCR0A = 195;					// Output Compare Match with prescaler 1/1024: Every ~10ms
	TCCR0A |= (1<<WGM01);			// Clear Timer on Compare match (CTC), Mode 2
	TCCR0B |= (1<<CS02) | (1<<CS00);				// Prescaler 1/1024
	TIMSK0 |= (1<<OCIE0A);			// Timer/Counter1 Output Compare Interrupt Request
}

 void Calibration_cycle(uint8_t *currPWMlvl, uint8_t *currDAClvl){
	uint8_t adclevel;
	set_DAC_level(3, currDAClvl);
	set_PGA(128, currPWMlvl);
	
	for (int i=0; i<8; i++){
		set_MUX(i);
		enable_MUX();
		_delay_ms(100);
			
		while (PGA_errorflag()){			// While PGA puts out Error Flag (Signal clips to supply rail) reduce Gain by one step
			_delay_ms(100);
			set_PGA(0, currPWMlvl);
		}
		
		adclevel = ADC_get_value();
		while (adclevel > 113){				// While Signal higher than 113 (=2.2V. 2.5V is the upper range of mainboard ADC)
			set_PGA(0, currPWMlvl);
			_delay_ms(100);
			adclevel = ADC_get_value();
		}
		disable_MUX();
	}	
	ADC_disable();
 }
	 
  
 void ADC_init(){
	ADMUX |= (1<<REFS0)|(1<<ADLAR)|(1<<MUX2)|(1<<MUX1)|(1<<MUX0);		// AVCC Reference with external capacitor at AREF; Left Adjust result (8MSBits from ADCH); ADC7 select
	ADCSRA |= (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)|(1<<ADSC)|(1<<ADATE);			// 1/ 128Prescaling bit for ~150kHz ADC clock, Single Ended Conversion; Auto Trigger Enable, Free Running Mode		// 
	ADCSRA |= (1<<ADEN)|(1<<ADSC);											// Enable ADC, Set ADC Start Conversion Bit	  
  }
  
  void ADC_disable(){
	ADCSRA &= ~(1<<ADEN);			// Disable ADC for Power Saving
  }
  
  uint8_t ADC_get_value(){
	  uint8_t val =0;
	  val = ADCH;
	  return val;
  }

 void set_DAC_level(uint8_t val, uint8_t *currDAClvl){		// sets DAC-Word (4 steps: 1-4)
	 switch (val){
		case 0:{					// reduce DAClvl by one step
			if (*currDAClvl == 4){
				PORTC = 0b10000000;	
				*currDAClvl =3;
			}
			if (*currDAClvl == 3){
				PORTC = 0b01111111;
				*currDAClvl =2;
			}
			if (*currDAClvl == 2){
				PORTC = 0b00000000;
				*currDAClvl =1;
			}
			if (*currDAClvl == 1){
				PORTC = 0b00000000;
				*currDAClvl =1;
			}
			break;
		} 
		case 1:{
			PORTC = 0b00000000;		// 440mV
			*currDAClvl =1;
			break;
		}
		case 2:{
			PORTC = 0b01111111;		// 960mV
			*currDAClvl =2;
			break;
		}
		case 3:{
			PORTC = 0b10000000;		// 1.44V
			*currDAClvl =3;
			break;
		}
		case 4:{
			PORTC = 0b11111111;		// 1.96V
			*currDAClvl =4;
			break;
		}
		 default: break;
	 }	 
 }
 
 
 void enable_MUX(){				// Enables MUX (E)
	 PORTA &= ~(1<<PINA0);
 }
 
 void disable_MUX(){
	 PORTA |= (1<<PINA0);
 }
 
 void set_MUX(uint8_t channelnumber){					// Sets MUX output channel
	 switch (channelnumber){
		 case 0:{
			PORTA &= ~((1<<PINA3) | (1<<PINA2) | (1<<PINA1));		// Y0: 0b000
			break;
		 }
		 case 1:{
			PORTA &= ~((1<<PINA2) | (1<<PINA1));					// Y1: 0b001
			PORTA |= (1<<PINA3);  
			break;
		 }
		 case 2:{
			PORTA &= ~((1<<PINA3) | (1<<PINA1));					// Y2: 0b010
			PORTA |= (1<<PINA2);
			break;
		 }
		 case 3:{
			PORTA &= ~(1<<PINA1);									// Y3: 0b011
			PORTA |= (1<<PINA3) | (1<<PINA2);
			break;
		 }
		 case 4:{
			PORTA &= ~((1<<PINA3) | (1<<PINA2));					// Y4: 0b100
			PORTA |= (1<<PINA1);
			break;
		 }		 
		 case 5:{
			PORTA &= ~(1<<PINA2);									// Y5: 0b101
			PORTA |= (1<<PINA3) | (1<<PINA1);
			break;
		 }
		 case 6:{
			PORTA &= ~(1<<PINA3);									// Y6: 0b110
			PORTA |= (1<<PINA2) | (1<<PINA1);
			break;
		 }		 
		 case 7:{
			PORTA |= ((1<<PINA3) | (1<<PINA2) | (1<<PINA1));		// Y7: 0b111
			break;
		 }
		 default:	break;
	 }

 }
 
 void PWM_init(){
	 //Initialize Lock-In-PWM-Signal
	 PRR0 &= ~(1<<PRTIM2);													// Power Reduction Register: Timer2 Enable
	 TCCR2A=(1<<COM2A0) | (1<< WGM20);										// WGM22:0 = 101 PWM, Phase Correct Mode; COM2A0=1 Toggle on Compare Match
	 TCCR2B=(1<<CS21) | (1<<WGM22);											// 1/8 prescaling,
	 //set_LOCK_PWM(62);														// PWM: 10,08 kHz, 50% Duty Cycle
	 //set_LOCK_PWM(250);													// PWM: 2,50 kHz
	 set_LOCK_PWM(200);														// PWM: 3,125 kHz
 }
 
 void enable_LOCK_PWM(){
	 TCCR2A |= (1<<COM2A0);						// Phase correct PWM Mode, Toggle OC1A on Compare Match
 }
 
 void disable_LOCK_PWM(){
	 TCCR2A &= ~(1<<COM2A0);					// Non PWM Mode, Normal port operation, OC1A disconnected 
 }
 
 void set_LOCK_PWM(uint8_t pwm_val){
	 OCR2A = pwm_val;
 }
 
 void set_PGA(uint8_t PGA_val, uint8_t *currPGAlvl){		// Available Gains: 1, 2, 4, 8, 16, 32, 64, 128 NOTE: *1.375 / 2 because of Single-Output-Use & G4=1
	  /*	
		PB4				Logic			44		LSB		G0
	  	PB3				Logic			43				G1
	  	PB2				Logic			42				G2
	  	PB1				Logic			41				G3
	  	PB0				Logic			40		MSB		G4*/
	  switch (PGA_val){
		  case 0:{					// Reduce Gain by one step
			if (*currPGAlvl ==128){
				PORTB = 0b00001001;
				*currPGAlvl = 64;
			}
			if (*currPGAlvl ==64){
				PORTB = 0b00001000;
				*currPGAlvl = 32;
			}
			if (*currPGAlvl ==32){
				PORTB = 0b00000111;
				*currPGAlvl = 16;
			}
			if (*currPGAlvl ==16){
				PORTB = 0b00000110;
				*currPGAlvl = 8;
			}
			if (*currPGAlvl ==8){
				PORTB = 0b00000101;
				*currPGAlvl = 4;
			}
			if (*currPGAlvl ==4){
				PORTB = 0b00000100;
				*currPGAlvl = 2;
			}
			if (*currPGAlvl ==2){
				PORTB = 0b00000011;
				*currPGAlvl = 1;
			}
			if (*currPGAlvl ==1){
				PORTB = 0b00000011;
				*currPGAlvl = 1;
			}  
			break;
		  }
		  case 1:{
			PORTB = 0b00010011;		// Gain = 0.688; G4 = 1, G3:G0= 0011, 
			*currPGAlvl=1;
			break;
		  }
		  case 2:{
			PORTB = 0b00010100;		// Gain = 1.375; G4 = 1, G3:G0= 0100
			*currPGAlvl=2;
			break;
		  }
		  case 4:{
			PORTB = 0b00010101;		// Gain = 2.75; G4 = 1, G3:G0= 0101
			*currPGAlvl=4;
			break;
		  }
		  case 8:{
			PORTB = 0b00010110;		// Gain = 5.5; G4 = 1, G3:G0= 0110
			*currPGAlvl=8;
			break;
		  }
		  case 16:{
			PORTB = 0b00010111;		// Gain = 11; G4 = 1, G3:G0= 0111
			*currPGAlvl=16;
			break;
		  }	  
		  case 32:{
			PORTB = 0b00011000;		// Gain = 22; G4 = 1, G3:G0= 1000
			*currPGAlvl=32;
			break;
		  }	  
		  case 64:{
			PORTB = 0b00011001;		// Gain = 44; G4 = 1, G3:G0= 1001
			*currPGAlvl=64;
			break;
		  }
		  case 128:{
			PORTB = 0b00011010;		// Gain = 88; G4 = 1, G3:G0= 1010
			*currPGAlvl=128;
			break;	
		  } 
		  default: break;
	  }  
 }
 
 bool PGA_errorflag(){
	 bool portflag = (PIND & 0x01);
	 return portflag;
 }
 
 void LED1_on(){
	 PORTA |= (1<<PINA4);
 }
 
 void LED1_off(){
	 PORTA &= ~(1<<PINA4);
 }
 
 void LED2_on(){
	 PORTA |= (1<<PINA5);
 }
 
 void LED2_off(){
	 PORTA &= ~(1<<PINA5);
 }
 
 
 void LED_Init(){
	 LED1_on();
	 LED2_on();
	 _delay_ms(500);
	 LED1_off();
	 LED2_off();
	 _delay_ms(500);
	 LED1_on();
	 LED2_on();
	 _delay_ms(500);
	 LED1_off();
	 LED2_off();
 }
 
 void test_PGA(uint8_t *currPGAval){
	LED1_on();
	set_PGA(1,currPGAval);
	_delay_ms(2000);
	LED1_off();
	
	LED2_on();
	set_PGA(2,currPGAval);
	_delay_ms(2000);
	LED2_off();

	LED1_on();
	set_PGA(4,currPGAval);
	_delay_ms(2000);
	LED1_off();
	
	LED2_on();
	set_PGA(8,currPGAval);
	_delay_ms(2000);
	LED2_off();
	
	LED1_on();
	set_PGA(16,currPGAval);
	_delay_ms(2000);
	LED1_off();
	
	LED2_on();
	set_PGA(32,currPGAval);
	_delay_ms(2000);
	LED2_off();
	
	LED1_on();
	set_PGA(64,currPGAval);
	_delay_ms(2000);
	LED1_off();
	
	LED2_on();
	set_PGA(128,currPGAval);
	_delay_ms(2000);
	LED1_off();					
 }
 
 void test_MUX(){
	LED1_on();
	set_MUX(0);
	enable_MUX();
	_delay_ms(1000);
	LED1_off();
	disable_MUX();
	
	LED2_on();
	set_MUX(1);
	enable_MUX();
	_delay_ms(1000);
	LED2_off();
	disable_MUX();
	
	LED1_on();
	set_MUX(2);
	enable_MUX();
	_delay_ms(1000);
	LED1_off();
	disable_MUX();
	
	LED2_on();
	set_MUX(3);
	enable_MUX();
	_delay_ms(1000);
	LED2_off();
	disable_MUX();
		
	LED1_on();
	set_MUX(4);
	enable_MUX();
	_delay_ms(1000);
	LED1_off();
	disable_MUX();
	
	LED2_on();
	set_MUX(5);
	enable_MUX();
	_delay_ms(1000);
	LED2_off();
	disable_MUX();
	
	LED1_on();
	set_MUX(6);
	enable_MUX();
	_delay_ms(1000);
	LED1_off();
	disable_MUX();
	
	LED2_on();
	set_MUX(7);
	enable_MUX();
	_delay_ms(1000);
	LED2_off();
	disable_MUX();		
		
 }
   
 void test_DAC(uint8_t *currDACval){
	  LED1_on();
	  set_DAC_level(1,currDACval);
	  _delay_ms(1000);
	  LED1_off();
	  
	  LED2_on();
	  set_DAC_level(2,currDACval);
	  _delay_ms(1000);
	  LED2_off();	  

	  LED1_on();
	  set_DAC_level(3,currDACval);
	  _delay_ms(1000);
	  LED1_off();

	  LED2_on();
	  set_DAC_level(4,currDACval);
	  _delay_ms(1000);
	  LED2_off();	  	  
 }

 
 
 