/*************************************************************************
	Author: Alexander von Lhmann	2014
   Hardware
   
   processor:   AtMega164A
   clock:		20Mhz external RC oscillator

 					Function		Pin		Word	Connected With
 	Signal Monitor
 	PA7				ADC7			30				Signal
 	
 	Debug LEDs
 	PA5				Logic			32				LED 2
 	PA4				Logic			33				LED 1
 	
 	Multiplexer
 	PA1				Logic			34		MSB		S3
 	PA2				Logic			35				S2
 	PA3				Logic			36		LSB		S1
 	PA0				Logic			37		Enable [H]	E
 	
 	DAC
 	PC7				Logic			26		MSB		D7
 	PC6				Logic			25				D6
 	PC5				Logic			24				D5
 	PC4				Logic			23				D4
 	PC3				Logic			22				D3
 	PC2				Logic			21				D2
 	PC1				Logic			20				D1
 	PC0				Logic			19		LSB		D0
 	
 	PGA
 	PB4				Logic			44		LSB		G4
 	PB3				Logic			43				G3
 	PB2				Logic			42				G2
 	PB1				Logic			41				G1
 	PB0				Logic			40		MSB		G0
	PD0				ErrorFlag		9				EF
 	
 	PWM Lock In
 	PD7				PWM, OC2A		16
 	
 	PWM DAC2 
 	PB6				PWM	2
 	
 	Mainboard-Control-Signal
	PD4				RXRST			13
 	PD3				RXR				12
 	PD2				RX2				11
 	PD1				RX1				10


*************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "fNIRS_EVAL_v4_functions.h"
#include <util/delay.h>

// Global Variable
volatile uint8_t vect_cnt;
volatile uint16_t timer_ms_counter;

ISR(INT1_vect){
	
	uint8_t portnr = ((PIND>>1)&0x03);		// Bitmask: Only PD1 + PD2 left over = Channelnumber
	switch(vect_cnt){
		case 0:{
			disable_MUX();
			timer_ms_counter =0;
			switch(portnr){
				case 0:{
					set_MUX(3);			// CH0 L0 750nm
					enable_MUX();
					break;
				}
				case 1:{
					set_MUX(1);			// CH1 L0 750nm	
					enable_MUX();
					break;
				}
				case 2:{
					set_MUX(7);			// CH2 L0 750nm	
					enable_MUX();
					break;
				}
				case 3:{
					set_MUX(6);			// CH3 L0 750nm	
					enable_MUX();
					break;
				}
			}
			vect_cnt=1;
			break;
		}
		case 1:{
			disable_MUX();
			timer_ms_counter =0;
			switch(portnr){
				case 0:{
					set_MUX(0);			// CH0 L1 850nm		
					enable_MUX();
					break;
				}
				case 1:{
					set_MUX(2);			// CH1 L1 850nm	
					enable_MUX();
					break;
				}
				case 2:{
					set_MUX(4);			// CH2 L1 850nm	
					enable_MUX();
					break;
				}
				case 3:{
					set_MUX(5);			// CH3 L1 850nm	
					enable_MUX();
					break;
				}
			}
			vect_cnt =0;
			break;
		}
	}
}

ISR(TIMER0_COMPA_vect) {
	if (timer_ms_counter > 500){
		timer_ms_counter =0;
		vect_cnt =0;
		disable_MUX();
	}
	else 
	timer_ms_counter++;
}

int main(void)
{
	
	/*********************** Initialization ***********************/
	
	//Port Directions
	
	DDRA= 0b00111111;														// MUX Control Signal (PA0-PA3, Output), LED Signal (PA4-PA5, Output), ADC7 Signal Monitor (PA7, Input)
	DDRB= 0b00011111;														// PGA Control Signal (PB0-PB4, Output)
	DDRC= 0xFF;																// DAC Control Signal (Output, 8Bit word)
	DDRD= 0b11000000;														// Control Signal from Mainboard (PD3-1), PWM Lock-In-Signal (PD7, Output), PGA Error Flag (PD0)
	
	uint8_t current_PGA_level;
	uint8_t current_DAC_level;
	
	//Initialize External Control Interrupt INT1on PD3
	EICRA |= (1<<ISC10) | (1<<ISC11);			// Rising edges of INT1 generate an IRequest
	EIMSK |= (1<<INT1);							// Enable INT1 
	
	PWM_init();
	ADC_init();
	Timer_init();
		
	_delay_ms(1000);
	//Calibration_cycle(&current_PGA_level, &current_DAC_level);
	set_DAC_level(4, &current_DAC_level);
	set_PGA(64, &current_PGA_level);
	
	vect_cnt =0;
	timer_ms_counter =0;
	
	test_MUX();
	disable_MUX();
	
	sei();				// Enable Interrupts
	LED_Init();			// Show RDY Signal

	
	set_MUX(1);
	enable_MUX();
	
	while(1)
    {	
		// reset?
		if (PIND & 0x10){
		vect_cnt =0;
		disable_MUX();
		}
    }
}